/*
 * Copyright 1994, Brown University, Providence, RI
 *
 * Permission to use and modify this software and its documentation for
 * any purpose other than its incorporation into a commercial product is
 * hereby granted without fee.  Permission to copy and distribute this
 * software and its documentation only for non-commercial use is also
 * granted without fee, provided, however, that the above copyright notice
 * appear in all copies, that both that copyright notice and this permission
 * notice appear in supporting documentation, that the name of Brown
 * University not be used in advertising or publicity pertaining to
 * distribution of the software without specific, written prior permission,
 * and that the person doing the distribution notify Brown University of
 * such distributions outside of his or her organization. Brown University
 * makes no representations about the suitability of this software for
 * any purpose.  It is provided "as is" without express or implied warranty.
 * Brown University requests notification of any modifications to this
 * software or its documentation.
 *
 * Send the following redistribution information:
 *
 *	Name:
 *	Organization:
 *	Address (postal and/or electronic):
 *
 * To:
 *	Software Librarian
 *	Computer Science Department, Box 1910
 *	Brown University
 *	Providence, RI 02912
 *
 *		or
 *
 *	brusd@cs.brown.edu
 *
 * We will acknowledge all electronic notifications.
 */

/*
**	xnull.c
**
**	Null X client.  Sustains an X server for use by another
**	(remote) client.  Control-q on default root quits.
**
**	WARNING:  disables access control by default
**
*/
#include <sys/types.h>
#include <sys/socket.h>
#include <netdb.h>

#include <stdio.h>
#include <X11/X.h>
#include <X11/Xlib.h>
#include <X11/keysym.h>

char *usage = "usage: xnull [-q] [hostname...]\n";

main(argc, argv)
   int argc;
   char **argv;
{
   int verbose = 1;
   Display *dpy;
   XSetWindowAttributes swa;
   XEvent event;
   XHostAddress ha;
   struct hostent *hp;

   for (argc--, argv++; argc && **argv == '-'; argc--, argv++)
      switch (*(*argv+1)) {
         case 'q':
            verbose = 0;
            break;
         default:
            fprintf(stderr, usage);
            exit(-1);
      }

   dpy = XOpenDisplay(0);
   if (dpy) {
      swa.event_mask = KeyPressMask;
      XChangeWindowAttributes(dpy, DefaultRootWindow(dpy), CWEventMask, &swa);
      if (argc)
         for (; argc--; argv++)
            if (hp = gethostbyname(*argv)) {
               ha.family = FamilyInternet;	/* so sue me */
               ha.length = hp->h_length;
               ha.address = *hp->h_addr_list;
               XAddHost(dpy, &ha);
               if (verbose)
                  printf("xnull: added %s to access list\n", hp->h_name);
            }
            else
               fprintf(stderr, "xnull: unknown host '%s'\n", *argv);
      else {
         XDisableAccessControl(dpy);
         if (verbose)
            printf("xnull: access control disabled\n");
      }
      while (1) {
         XNextEvent(dpy, &event);
         switch (event.type) {
            case KeyPress:
               if (	XKeycodeToKeysym(dpy, event.xkey.keycode, 0) == XK_q &&
			event.xkey.state & ControlMask)
                  exit(0);
               break;
         }
      }

   }
}
